@@ -1,12 +1,13 @@
 #include "version.h"
 #include "cpuid.h"
 #include <stdio.h>
+#include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 
-__ID("@(#) $Id: cpuid.cc 1726 2007-02-13 21:38:11Z lyonel $");
+__ID("@(#) $Id: cpuid.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 #if defined(__i386__) || defined(__alpha__)
 
@@ -72,8 +73,13 @@ int n = 0)
 
 #ifdef __i386__
 
+/* %ebx may be the PIC register.  */
 #define cpuid_up(in,a,b,c,d)\
-asm("cpuid": "=a" (a), "=b" (b), "=c" (c), "=d" (d) : "a" (in));
+  __asm__ ("xchgl\t%%ebx, %1\n\t"			\
+	   "cpuid\n\t"					\
+	   "xchgl\t%%ebx, %1\n\t"			\
+	   : "=a" (a), "=r" (b), "=c" (c), "=d" (d)	\
+	   : "0" (in))
 
 static void cpuid(int cpunumber,
 unsigned long idx,
@@ -92,12 +98,14 @@ unsigned long &edx)
   {
     lseek(fd, idx, SEEK_CUR);
     memset(buffer, 0, sizeof(buffer));
-    read(fd, buffer, sizeof(buffer));
+    if(read(fd, buffer, sizeof(buffer)) == sizeof(buffer))
+    {
+      eax = (*(unsigned long *) buffer);
+      ebx = (*(unsigned long *) (buffer + 4));
+      ecx = (*(unsigned long *) (buffer + 8));
+      edx = (*(unsigned long *) (buffer + 12));
+    }
     close(fd);
-    eax = (*(unsigned long *) buffer);
-    ebx = (*(unsigned long *) (buffer + 4));
-    ecx = (*(unsigned long *) (buffer + 8));
-    edx = (*(unsigned long *) (buffer + 12));
   }
   else
     cpuid_up(idx, eax, ebx, ecx, edx);
