@@ -4,18 +4,22 @@
 #include "version.h"
 #include "options.h"
 #include "heuristics.h"
+#include <cstring>
 #include <vector>
 #include <map>
 #include <sstream>
-#include <unistd.h>
+#include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <ctype.h>
+#include <limits.h>
 #include <unistd.h>
+#include <limits.h>
 #include <sys/utsname.h>
 
 using namespace hw;
 
-__ID("@(#) $Id: hw.cc 1907 2007-10-13 22:23:16Z lyonel $");
+__ID("@(#) $Id: hw.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 struct hwNode_i
 {
