@@ -41,11 +41,12 @@
 #include "ideraid.h"
 #include "mounts.h"
 #include "smp.h"
+#include "abi.h"
 
 #include <unistd.h>
 #include <stdio.h>
 
-__ID("@(#) $Id: main.cc 1907 2007-10-13 22:23:16Z lyonel $");
+__ID("@(#) $Id: main.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 bool scan_system(hwNode & system)
 {
@@ -132,6 +133,9 @@ bool scan_system(hwNode & system)
     status("CPUFreq");
     if (enabled("cpufreq"))
       scan_cpufreq(computer);
+    status("ABI");
+    if (enabled("abi"))
+      scan_abi(computer);
     status("");
 
     if (computer.getDescription() == "")
