@@ -4,6 +4,7 @@
 #include <string>
 #include <vector>
 #include <sys/types.h>
+#include <stdint.h>
 
 bool pushd(const std::string & dir = "");
 std::string popd();
@@ -26,6 +27,7 @@ std::string get_devid(const std::string &);
 std::string uppercase(const std::string &);
 std::string lowercase(const std::string &);
 std::string tostring(unsigned long long);
+std::string tohex(unsigned long long);
 std::string utf8(wchar_t);
 std::string utf8_sanitize(const std::string &);
 std::string utf8(uint16_t *s, size_t length = -1, bool forcelittleendian = false);
@@ -43,6 +45,7 @@ bool matches(const std::string & s, const std::string & pattern, int cflags=0);
 
 int selectdir(const struct dirent *d);
 int selectlink(const struct dirent *d);
+int selectfile(const struct dirent *d);
 
 unsigned short be_short(const void *);
 unsigned short le_short(const void *);
