@@ -17,10 +17,12 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 #include <dirent.h>
 
-__ID("@(#) $Id: parisc.cc 1907 2007-10-13 22:23:16Z lyonel $");
+__ID("@(#) $Id: parisc.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 #define DEVICESPARISC "/sys/devices/parisc"
 
