@@ -18,15 +18,18 @@
 #include "partitions.h"
 #include "blockio.h"
 #include "lvm.h"
+#include "volumes.h"
 #include "osutils.h"
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <string.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <stdint.h>
 
-__ID("@(#) $Id: partitions.cc 1726 2007-02-13 21:38:11Z lyonel $");
+__ID("@(#) $Id: partitions.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 #define LIFBLOCKSIZE 256
 
@@ -40,15 +43,17 @@ struct maptypes
 static bool detect_dosmap(source & s, hwNode & n);
 static bool detect_macmap(source & s, hwNode & n);
 static bool detect_lif(source & s, hwNode & n);
+static bool detect_luks(source & s, hwNode & n);
 static bool detect_gpt(source & s, hwNode & n);
 
 static struct maptypes map_types[] =
 {
-  {"mac", "Apple Macintosh partition map", detect_macmap},
   {"bsd", "BSD disklabel", NULL},
   {"gpt", "GUID partition table", detect_gpt},
   {"dos", "MS-DOS partition table", detect_dosmap},
+  {"mac", "Apple Macintosh partition map", detect_macmap},
   {"lif", "HP-UX LIF", detect_lif},
+  {"luks", "Linux Unified Key Setup", detect_luks},
   {"solaris-x86", "Solaris disklabel", NULL},
   {"solaris-sparc", "Solaris disklabel", NULL},
   {"raid", "Linux RAID", NULL},
@@ -58,44 +63,6 @@ static struct maptypes map_types[] =
   { NULL, NULL, NULL }
 };
 
-struct fstypes
-{
-  const char * id;
-  const char * description;
-  const char * capabilities;
-};
-
-static struct fstypes fs_types[] =
-{
-  {"blank", "Blank", ""},
-  {"fat", "MS-DOS FAT derivatives (FAT12, FAT16, FAT32)", ""},
-  {"ntfs", "Windows NTFS", "secure"},
-  {"hpfs", "OS/2 HPFS", "secure"},
-  {"ext2", "Linux EXT2", "secure"},
-  {"ext3", "Linux EXT3", "secure,journaled"},
-  {"reiserfs", "Linux ReiserFS", "secure,journaled"},
-  {"romfs", "Linux ROMFS", "ro"},
-  {"squashfs", "Linux SquashFS", "ro"},
-  {"cramfs", "Linux CramFS", "ro"},
-  {"minixfs", "MinixFS", "secure"},
-  {"sysvfs", "System V FS", "secure"},
-  {"jfs", "Linux JFS", "secure,journaled"},
-  {"xfs", "Linux XFS", "secure,journaled"},
-  {"iso9660", "ISO-9660", "secure,ro"},
-  {"xboxdvd", "X-Box DVD", "ro"},
-  {"udf", "UDF", "secure,ro"},
-  {"ufs", "UFS", "secure"},
-  {"hphfs", "HP-UX HFS", "secure"},
-  {"vxfs", "VxFS", "secure,journaled"},
-  {"ffs", "FFS", "secure"},
-  {"befs", "BeOS BFS", "journaled"},
-  {"qnxfs", "QNX FS", ""},
-  {"mfs", "MacOS MFS", ""},
-  {"hfs", "MacOS HFS", ""},
-  {"hfsplus", "MacOS HFS+", "secure,journaled"},
-  { NULL, NULL, NULL }
-};
-
 struct efi_guid_t
 {
   uint32_t time_low;
@@ -346,9 +313,10 @@ static bool guess_logicalname(source & s, const hwNode & disk, unsigned int n, h
 
     spart.offset = 0;
     spart.fd = dev;
+    spart.diskname = "";
 
+    // read the first sector
     if((readlogicalblocks(s, buffer1, 0, 1)!=1) ||
-                                                  // read the first sector
       (readlogicalblocks(spart, buffer2, 0, 1)!=1))
     {
       close(dev);
@@ -1010,7 +978,7 @@ static bool detect_gpt(source & s, hwNode & n)
       spart.offset = s.offset + p.StartingLBA*spart.blocksize;
       spart.size = (p.EndingLBA - p.StartingLBA)*spart.blocksize;
       guess_logicalname(spart, n, i+1, partition);
-      scan_lvm(partition, spart);
+      scan_volume(partition, spart);
       n.addChild(partition);
     }
   }
@@ -1029,6 +997,7 @@ static bool detect_dosmap(source & s, hwNode & n)
   unsigned char type;
   unsigned long long start, size;
   bool partitioned = false;
+  unsigned long signature = 0;
 
   if(s.offset!=0)
     return false;                                 // partition tables must be at the beginning of the disk
@@ -1039,6 +1008,16 @@ static bool detect_dosmap(source & s, hwNode & n)
   if(le_short(buffer+510)!=0xaa55)                // wrong magic number
     return false;
 
+  signature=le_long(buffer+440);
+  if(signature == 0xffffffffL)
+    signature = 0;
+  if(signature)
+  {
+    char buffer[8+1];
+    snprintf(buffer, sizeof(buffer), "%08lx", signature);
+    n.setConfig("signature", buffer);
+  }
+
   lastlogicalpart = 5;
 
   for(i=0; i<4; i++)
@@ -1066,6 +1045,7 @@ static bool detect_dosmap(source & s, hwNode & n)
     if(analyse_dospart(spart, flags, type, partition))
     {
       guess_logicalname(spart, n, i+1, partition);
+      scan_volume(partition, spart);
       n.addChild(partition);
       partitioned = true;
     }
@@ -1133,7 +1113,7 @@ static bool detect_macmap(source & s, hwNode & n)
 
       guess_logicalname(spart, n, i, partition);
 
-      scan_lvm(partition, spart);
+      scan_volume(partition, spart);
       n.addChild(partition);
     }
   }
@@ -1191,6 +1171,36 @@ static bool detect_lif(source & s, hwNode & n)
   return true;
 }
 
+static bool detect_luks(source & s, hwNode & n)
+{
+  static char buffer[BLOCKSIZE];
+  source luksvolume;
+  unsigned luks_version = 0;
+
+  luksvolume = s;
+  luksvolume.blocksize = BLOCKSIZE;
+                                                  // read the first block
+  if(readlogicalblocks(luksvolume, buffer, 0, 1)!=1)
+    return false;
+
+  if(memcmp(buffer, "LUKS", 4) != 0)                    // wrong magic number
+    return false;
+  if(be_short(buffer+4) != 0xbabe)
+    return false;
+
+  luks_version = be_short(buffer+6);
+  if(luks_version<1)
+    return false;                 // weird LUKS version
+  else
+  {
+    hwNode partition("volume", hw::volume);
+    scan_volume(partition, luksvolume);
+    partition.setLogicalName(n.getLogicalName());
+    n.addChild(partition);
+  }
+
+  return true;
+}
 
 bool scan_partitions(hwNode & n)
 {
@@ -1204,7 +1214,7 @@ bool scan_partitions(hwNode & n)
 
   if(n.isCapable("removable"))
   {
-    medium = n.addChild(hwNode("disc", hw::disk));
+    medium = n.addChild(hwNode("medium", hw::disk));
 
     medium->claim();
     medium->setSize(n.getSize());
@@ -1233,7 +1243,8 @@ bool scan_partitions(hwNode & n)
 
   if(!medium->isCapable("partitioned"))
   {
-    scan_lvm(*medium, s);
+    if(scan_volume(*medium, s))	// whole disk volume?
+      medium->setClass(hw::volume);
   }
 
   close(fd);
