@@ -1,14 +1,18 @@
 #include "version.h"
 #include "pci.h"
 #include "osutils.h"
+#include "options.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <stdint.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
 #include <dirent.h>
 
-__ID("@(#) $Id: pci.cc 1907 2007-10-13 22:23:16Z lyonel $");
+__ID("@(#) $Id: pci.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 #define PROC_BUS_PCI "/proc/bus/pci"
 #define SYS_BUS_PCI "/sys/bus/pci"
@@ -66,6 +70,7 @@ __ID("@(#) $Id: pci.cc 1907 2007-10-13 22:23:16Z lyonel $");
 #define PCI_CAP_LIST_NEXT       1       /* Next capability in the list */
 #define PCI_CAP_FLAGS           2       /* Capability defined flags (16 bits) */
 #define PCI_CAP_SIZEOF          4
+#define PCI_FIND_CAP_TTL       48
 
 #define PCI_SID_ESR             2       /* Expansion Slot Register */
 #define  PCI_SID_ESR_NSLOTS     0x1f    /* Number of expansion slots available */
@@ -664,8 +669,9 @@ static bool scan_capabilities(hwNode & n, struct pci_dev &d)
 {
   unsigned int where = get_conf_byte(d, PCI_CAPABILITY_LIST) & ~3;
   string buffer;
+  unsigned int ttl = PCI_FIND_CAP_TTL;
 
-  while(where)
+  while(where && ttl--)
   {
     unsigned int id, next, cap;
 
@@ -673,7 +679,7 @@ static bool scan_capabilities(hwNode & n, struct pci_dev &d)
     next = get_conf_byte(d, where + PCI_CAP_LIST_NEXT) & ~3;
     cap = get_conf_word(d, where + PCI_CAP_FLAGS);
 
-    if(!id)
+    if(!id || id == 0xff)
       return false;
 
     switch(id)
@@ -799,8 +805,8 @@ static hwNode *scan_pci_dev(struct pci_dev &d, hwNode & n)
           hw::bridge);
 
         host.setDescription(get_class_description(dclass, progif));
-        host.setVendor(get_device_description(d.vendor_id));
-        host.setProduct(get_device_description(d.vendor_id, d.device_id));
+        host.setVendor(get_device_description(d.vendor_id)+(enabled("output:numeric")?" ["+tohex(d.vendor_id)+"]":""));
+        host.setProduct(get_device_description(d.vendor_id, d.device_id)+(enabled("output:numeric")?" ["+tohex(d.vendor_id)+":"+tohex(d.device_id)+"]":""));
         host.setHandle(pci_bushandle(d.bus, d.domain));
         host.setVersion(revision);
         addHints(host, d.vendor_id, d.device_id, subsys_v, subsys_d, dclass);
@@ -946,9 +952,9 @@ static hwNode *scan_pci_dev(struct pci_dev &d, hwNode & n)
           {
             device->addCapability(moredescription);
           }
-          device->setVendor(get_device_description(d.vendor_id));
+          device->setVendor(get_device_description(d.vendor_id)+(enabled("output:numeric")?" ["+tohex(d.vendor_id)+"]":""));
           device->setVersion(revision);
-          device->setProduct(get_device_description(d.vendor_id, d.device_id));
+          device->setProduct(get_device_description(d.vendor_id, d.device_id)+(enabled("output:numeric")?" ["+tohex(d.vendor_id)+":"+tohex(d.device_id)+"]":""));
 
           if (cmd & PCI_COMMAND_MASTER)
             device->addCapability("bus master", "bus mastering");
@@ -982,7 +988,7 @@ static hwNode *scan_pci_dev(struct pci_dev &d, hwNode & n)
             else
               result = n.addChild(*device);
           }
-          free(device);
+          delete device;
 
         }
       }
@@ -1055,7 +1061,8 @@ bool scan_pci_legacy(hwNode & n)
       fd = open(devicepath.c_str(), O_RDONLY);
       if (fd >= 0)
       {
-        read(fd, d.config, sizeof(d.config));
+        if(read(fd, d.config, sizeof(d.config)) != sizeof(d.config))
+          memset(&d.config, 0, sizeof(d.config));
         close(fd);
       }
 
@@ -1102,8 +1109,11 @@ bool scan_pci(hwNode & n)
       if (fd >= 0)
       {
         memset(&d, 0, sizeof(d));
-        read(fd, d.config, 64);
-        read(fd, d.config+64, sizeof(d.config)-64);
+        if(read(fd, d.config, 64) == 64)
+        {
+          if(read(fd, d.config+64, sizeof(d.config)-64) != sizeof(d.config)-64)
+            memset(d.config+64, 0, sizeof(d.config)-64);
+        }
         close(fd);
       }
 
