@@ -11,15 +11,17 @@
 #include "options.h"
 #include "version.h"
 #include "osutils.h"
+#include <cstring>
 #include <iostream>
 #include <sstream>
 #include <iomanip>
 #include <unistd.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <sys/ioctl.h>
 #include <termios.h>
 
-__ID("@(#) $Id: print.cc 1907 2007-10-13 22:23:16Z lyonel $");
+__ID("@(#) $Id: print.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 static unsigned int columns = 0, rows = 0;
 
