@@ -5,6 +5,7 @@
 #include "osutils.h"
 #include "heuristics.h"
 #include "sysfs.h"
+#include <glob.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -12,16 +13,21 @@
 #include <unistd.h>
 #include <dirent.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <scsi/sg.h>
 #include <scsi/scsi.h>
 #ifndef MKDEV
 #include <linux/kdev_t.h>
 #endif
+#ifndef MINOR
+#include <linux/kdev_t.h>
+#endif
 
 #include <string>
 #include <map>
 
-__ID("@(#) $Id: scsi.cc 1907 2007-10-13 22:23:16Z lyonel $");
+__ID("@(#) $Id: scsi.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 #define SG_X "/dev/sg%d"
 #define SG_MAJOR 21
@@ -184,27 +190,15 @@ My_scsi_idlun;
 
 static const char *devices[] =
 {
-  "/dev/sda", "/dev/sdb", "/dev/sdc", "/dev/sdd", "/dev/sde", "/dev/sdf",
-  "/dev/sdg", "/dev/sdh", "/dev/sdi", "/dev/sdj", "/dev/sdk", "/dev/sdl",
-  "/dev/sdm", "/dev/sdn", "/dev/sdo", "/dev/sdp", "/dev/sdq", "/dev/sdr",
-  "/dev/sds", "/dev/sdt", "/dev/sdu", "/dev/sdv", "/dev/sdw", "/dev/sdx",
-  "/dev/sdy", "/dev/sdz", "/dev/sdaa", "/dev/sdab", "/dev/sdac", "/dev/sdad",
-  "/dev/scd0", "/dev/scd1", "/dev/scd2", "/dev/scd3", "/dev/scd4",
-  "/dev/scd5",
-  "/dev/scd6", "/dev/scd7", "/dev/scd8", "/dev/scd9", "/dev/scd10",
-  "/dev/scd11", "/dev/sr0", "/dev/sr1", "/dev/sr2", "/dev/sr3", "/dev/sr4",
-  "/dev/sr5",
-  "/dev/sr6", "/dev/sr7", "/dev/sr8", "/dev/sr9", "/dev/sr10", "/dev/sr11",
-  "/dev/cdrom", "/dev/cdrom0", "/dev/cdrom1", "/dev/cdrom2",
-  "/dev/cdwriter", "/dev/cdwriter0", "/dev/cdwriter1", "/dev/cdwriter2",
-  "/dev/dvd", "/dev/dvd0", "/dev/dvd1", "/dev/dvd2",
-  "/dev/dvdwriter", "/dev/dvdwriter0", "/dev/dvdwriter1", "/dev/dvdwriter2",
-  "/dev/st0", "/dev/st1", "/dev/st2", "/dev/st3", "/dev/st4", "/dev/st5",
-  "/dev/nst0", "/dev/nst1", "/dev/nst2", "/dev/nst3", "/dev/nst4",
-  "/dev/nst5",
-  "/dev/nosst0", "/dev/nosst1", "/dev/nosst2", "/dev/nosst3", "/dev/nosst4",
-  "/dev/tape", "/dev/tape0", "/dev/tape1", "/dev/tape2", "/dev/tape3",
-  "/dev/tape4",
+  "/dev/sd*[!0-9]",	/* don't look at partitions */
+  "/dev/scd*",
+  "/dev/sr*",
+  "/dev/cd*",
+  "/dev/dvd*",
+  "/dev/st*",
+  "/dev/nst*",
+  "/dev/nosst*",
+  "/dev/tape*",
   NULL
 };
 
@@ -446,7 +440,6 @@ static bool do_inquiry(int sg_fd,
 hwNode & node)
 {
   char rsp_buff[MX_ALLOC_LEN + 1];
-  char version[5];
   int k;
   unsigned int len;
 
@@ -465,6 +458,8 @@ hwNode & node)
     if (!do_inq(sg_fd, 0, 0, 0, rsp_buff, len, 1))
       return false;
   }
+  else
+    return false;
 
   if (len != ((unsigned int) rsp_buff[4] + 5))
     return false;                                 // twin INQUIRYs yield different lengths
@@ -480,9 +475,8 @@ hwNode & node)
   if (len > 32)
     node.setVersion(string(rsp_buff + 32, 4));
 
-  snprintf(version, sizeof(version), "%d", ansiversion);
   if (ansiversion)
-    node.setConfig("ansiversion", version);
+    node.setConfig("ansiversion", tostring(ansiversion));
 
   memset(rsp_buff, 0, sizeof(rsp_buff));
   if (do_inq(sg_fd, 0, 1, 0x80, rsp_buff, MX_ALLOC_LEN, 0))
@@ -551,37 +545,50 @@ hwNode & node)
   return true;
 }
 
-
 static void scan_devices()
 {
   int fd = -1;
   int i = 0;
+  size_t j = 0;
   My_scsi_idlun m_idlun;
 
   for (i = 0; devices[i] != NULL; i++)
   {
-    fd = open(devices[i], O_RDONLY | O_NONBLOCK);
-    if (fd >= 0)
+    glob_t entries;
+
+    if(glob(devices[i], 0, NULL, &entries) == 0)
     {
-      int bus = -1;
-      char host[50];
-      int * length = (int*)host;
-      *length = sizeof(host);
-      memset(host, 0, sizeof(host));
-      if (ioctl(fd, SCSI_IOCTL_PROBE_HOST, &host) >= 0)
+      for(j=0; j < entries.gl_pathc; j++)
       {
-        if (ioctl(fd, SCSI_IOCTL_GET_BUS_NUMBER, &bus) >= 0)
+        fd = open(entries.gl_pathv[j], O_RDONLY | O_NONBLOCK);
+        if (fd >= 0)
         {
-          memset(&m_idlun, 0, sizeof(m_idlun));
-          if (ioctl(fd, SCSI_IOCTL_GET_IDLUN, &m_idlun) >= 0)
+          int bus = -1;
+          union
           {
-            sg_map[string(devices[i])] = scsi_handle(bus, (m_idlun.mux4 >> 16) & 0xff,
-              m_idlun.mux4 & 0xff,
-              (m_idlun.mux4 >> 8) & 0xff);
+            char host[50];
+            int length;
+          } tmp;
+          tmp.length = sizeof(tmp.host);
+          memset(tmp.host, 0, sizeof(tmp.host));
+
+          if(ioctl(fd, SCSI_IOCTL_PROBE_HOST, &tmp.length) >= 0)
+          {
+            if (ioctl(fd, SCSI_IOCTL_GET_BUS_NUMBER, &bus) >= 0)
+            {
+              memset(&m_idlun, 0, sizeof(m_idlun));
+              if (ioctl(fd, SCSI_IOCTL_GET_IDLUN, &m_idlun) >= 0)
+              {
+                sg_map[string(entries.gl_pathv[j])] = scsi_handle(bus, (m_idlun.mux4 >> 16) & 0xff,
+                  m_idlun.mux4 & 0xff,
+                  (m_idlun.mux4 >> 8) & 0xff);
+              }
+            }
           }
+          close(fd);
         }
       }
-      close(fd);
+      globfree(&entries);
     }
   }
 }
@@ -607,28 +614,20 @@ int channel = -1,
 int target = -1,
 int lun = -1)
 {
-  char businfo[20];
   string result;
 
-  snprintf(businfo, sizeof(businfo), "scsi@%d", host);
-
-  result = string(businfo);
+  result = "scsi@" + tostring(host);
 
   if (channel >= 0)
   {
-    snprintf(businfo, sizeof(businfo), ":%d", channel);
-    result += string(businfo);
+    result += ":" + tostring(channel);
 
     if (target >= 0)
     {
-      snprintf(businfo, sizeof(businfo), ".%d", target);
-      result += string(businfo);
+      result += "." + tostring(target);
 
       if (lun >= 0)
-      {
-        snprintf(businfo, sizeof(businfo), ".%d", lun);
-        result += string(businfo);
-      }
+        result += "." + tostring(lun);
     }
   }
 
@@ -638,21 +637,13 @@ int lun = -1)
 
 static string host_logicalname(int i)
 {
-  char host[20];
-
-  snprintf(host, sizeof(host), "scsi%d", i);
-
-  return string(host);
+  return "scsi"+tostring(i);
 }
 
 
 static string host_kname(int i)                   // for sysfs
 {
-  char host[20];
-
-  snprintf(host, sizeof(host), "host%d", i);
-
-  return string(host);
+  return "host"+tostring(i);
 }
 
 
