@@ -11,6 +11,7 @@
 #include "usb.h"
 #include "osutils.h"
 #include "heuristics.h"
+#include "options.h"
 #include <stdio.h>
 #include <map>
 #include <sys/types.h>
@@ -18,6 +19,8 @@
 #include <sys/ioctl.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <string.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <dirent.h>
 
@@ -64,7 +67,7 @@ static map<u_int32_t,string> usbproducts;
 
 #define PRODID(x, y) ((x << 16) + y)
 
-__ID("@(#) $Id: usb.cc 1907 2007-10-13 22:23:16Z lyonel $");
+__ID("@(#) $Id: usb.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 static string usbhost(unsigned bus)
 {
@@ -274,12 +277,12 @@ static bool describeUSB(hwNode & device, unsigned vendor, unsigned prodid)
 {
   if(usbvendors.find(vendor)==usbvendors.end()) return false;
 
-  device.setVendor(usbvendors[vendor]);
+  device.setVendor(usbvendors[vendor]+(enabled("output:numeric")?" ["+tohex(vendor)+"]":""));
   device.addHint("usb.idVendor", vendor);
   device.addHint("usb.idProduct", prodid);
 
   if(usbproducts.find(PRODID(vendor, prodid))!=usbproducts.end())
-    device.setProduct(usbproducts[PRODID(vendor, prodid)]);
+    device.setProduct(usbproducts[PRODID(vendor, prodid)]+(enabled("output:numeric")?" ["+tohex(vendor)+":"+tohex(prodid)+"]":""));
 
   return true;
 }
@@ -348,14 +351,14 @@ bool scan_usb(hwNode & n)
   bool defined = false;
   unsigned int bus,lev,prnt,port,cnt,devnum,mxch;
   float speed;
-  char ver[10];
+  char ver[10+1];
   unsigned int cls, sub, prot, mxps, numcfgs;
   unsigned int vendor, prodid;
-  char rev[10];
+  char rev[10+1];
   unsigned numifs, cfgnum, atr;
-  char mxpwr[10];
+  char mxpwr[10+1];
   unsigned ifnum, alt, numeps;
-  char driver[80];
+  char driver[80+1];
 
   if (!exists(PROCBUSUSBDEVICES))
     return false;
@@ -374,8 +377,8 @@ bool scan_usb(hwNode & n)
   {
     char * buffer = NULL;
     size_t linelen;
-    char strname[80];
-    char strval[80];
+    char strname[80+1];
+    char strval[80+1];
 
     if(getline(&buffer, &linelen, usbdevices)>0)
     {
@@ -441,7 +444,7 @@ bool scan_usb(hwNode & n)
           case 'P':
             vendor = prodid = 0;
             strcpy(rev, "");
-            if(sscanf(line.c_str(), "P: Vendor=%x ProdID=%x Rev=%s", &vendor, &prodid, rev)>0)
+            if(sscanf(line.c_str(), "P: Vendor=%x ProdID=%x Rev=%10s", &vendor, &prodid, rev)>0)
             {
               describeUSB(device, vendor, prodid);
               device.setVersion(hw::strip(rev));
@@ -450,12 +453,12 @@ bool scan_usb(hwNode & n)
           case 'S':
             memset(strname, 0, sizeof(strname));
             memset(strval, 0, sizeof(strval));
-            if(sscanf(line.c_str(), "S: %[^=]=%[ -z]", strname, strval)>0)
+            if(sscanf(line.c_str(), "S: %80[^=]=%80[ -z]", strname, strval)>0)
             {
               if(strcasecmp(strname, "Manufacturer")==0)
-                device.setVendor(hw::strip(strval));
+                device.setVendor(hw::strip(strval)+(enabled("output:numeric")?" ["+tohex(vendor)+"]":""));
               if(strcasecmp(strname, "Product")==0)
-                device.setProduct(hw::strip(strval));
+                device.setProduct(hw::strip(strval)+(enabled("output:numeric")?" ["+tohex(vendor)+":"+tohex(prodid)+"]":""));
               if(strcasecmp(strname, "SerialNumber")==0)
                 device.setSerial(hw::strip(strval));
             }
@@ -465,13 +468,14 @@ bool scan_usb(hwNode & n)
             strcpy(mxpwr, "");
             if(sscanf(line.c_str(), "C:* #Ifs=%u Cfg#=%u Atr=%x MxPwr=%s", &numifs, &cfgnum, &atr, mxpwr)>0)
             {
-              device.setConfig("maxpower", mxpwr);
+              if(strcmp("0mA", mxpwr)!=0)
+                device.setConfig("maxpower", mxpwr);
             }
             break;
           case 'I':
             ifnum = alt = numeps = cls = sub = prot = 0;
             memset(driver, 0, sizeof(driver));
-            if((sscanf(line.c_str(), "I: If#=%u Alt=%u #EPs=%u Cls=%x(%*5c) Sub=%x Prot=%x Driver=%[ -z]", &ifnum, &alt, &numeps, &cls, &sub, &prot, driver)>0) && (cfgnum>0))
+            if(((sscanf(line.c_str(), "I:* If#=%u Alt=%u #EPs=%u Cls=%x(%*5c) Sub=%x Prot=%x Driver=%80[ -z]", &ifnum, &alt, &numeps, &cls, &sub, &prot, driver)>0) && (cfgnum>0)) || ((sscanf(line.c_str(), "I: If#=%u Alt=%u #EPs=%u Cls=%x(%*5c) Sub=%x Prot=%x Driver=%80[ -z]", &ifnum, &alt, &numeps, &cls, &sub, &prot, driver)>0) && (cfgnum>0)))
             {
               setUSBClass(device, cls, sub, prot);
               if((strlen(driver)!=0) && (strcasecmp("(none)", driver)!=0))
