@@ -9,11 +9,12 @@
 #include "support.h"
 #include "version.h"
 #include "engine.h"
+#include <string.h>
 
 static GtkWidget *about = NULL;
 extern GtkWidget *mainwindow;
 
-static char *id = "@(#) $Id: callbacks.c 1726 2007-02-13 21:38:11Z lyonel $";
+static char *id = "@(#) $Id: callbacks.c 1959 2008-05-09 20:32:50Z lyonel $";
 
 void
 refresh_display                        (GtkMenuItem     *menuitem,
@@ -50,7 +51,35 @@ void
 on_version_realize                     (GtkWidget       *widget,
 gpointer         user_data)
 {
+  const char *latest = checkupdates();
+
   gtk_label_set_text(GTK_LABEL(widget), getpackageversion());
+
+  if(latest)
+  {
+    if(strcmp(latest, getpackageversion()) != 0)
+    {
+      static GtkWidget *dialog = NULL;
+
+      if(!GTK_IS_WIDGET(dialog))
+      {
+        dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(mainwindow),
+                                  GTK_DIALOG_DESTROY_WITH_PARENT,
+                                  GTK_MESSAGE_INFO,
+                                  GTK_BUTTONS_CLOSE,
+                                  "The latest version is <tt>%s</tt>.\n\nYou can visit <span foreground=\"blue\"><u>http://www.ezix.org/</u></span> for more information.",
+                                  latest);
+
+        gtk_window_set_title(GTK_WINDOW(dialog), "Update available");
+        /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
+        g_signal_connect_swapped (dialog, "response",
+                           G_CALLBACK (gtk_widget_destroy),
+                           dialog);
+      }
+
+      gtk_widget_show(dialog);
+    }
+  }
 }
 
 
@@ -124,3 +153,19 @@ gpointer         user_data)
 {
   refresh(mainwindow);
 }
+
+void
+on_save_activate                       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  save_as(mainwindow);
+}
+
+
+void
+on_savebutton_clicked                  (GtkToolButton   *toolbutton,
+                                        gpointer         user_data)
+{
+  on_save_activate(NULL, NULL);
+}
+
