@@ -9,9 +9,11 @@
 #include <iostream>
 #include <fstream>
 #include <sys/utsname.h>
+#include <stdlib.h>
+#include <string.h>
 #include <libgen.h>
 
-static char *id = "@(#) $Id: engine.cc 1907 2007-10-13 22:23:16Z lyonel $";
+static char *id = "@(#) $Id: engine.cc 1959 2008-05-09 20:32:50Z lyonel $";
 
 extern "C"
 {
