@@ -8,9 +8,11 @@
 
 #include <unistd.h>
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
 #include <iostream>
 
-__ID("@(#) $Id: lshw.cc 1907 2007-10-13 22:23:16Z lyonel $");
+__ID("@(#) $Id: lshw.cc 1959 2008-05-09 20:32:50Z lyonel $");
 
 void usage(const char *progname)
 {
@@ -30,12 +32,14 @@ void usage(const char *progname)
   fprintf(stderr,
     "\t-class CLASS    only show a certain class of hardware\n");
   fprintf(stderr, "\t-C CLASS        same as '-class CLASS'\n");
+  fprintf(stderr, "\t-c CLASS        same as '-class CLASS'\n");
   fprintf(stderr,
     "\t-disable TEST   disable a test (like pci, isapnp, cpuid, etc. )\n");
   fprintf(stderr,
     "\t-enable TEST    enable a test (like pci, isapnp, cpuid, etc. )\n");
   fprintf(stderr, "\t-quiet          don't display status\n");
   fprintf(stderr, "\t-sanitize       sanitize output (remove sensitive information like serial numbers, etc.)\n");
+  fprintf(stderr, "\t-numeric        output numeric IDs (for PCI, USB, etc.)\n");
   fprintf(stderr, "\n");
 }
 
@@ -71,6 +75,7 @@ char **argv)
   disable("output:X");
   disable("output:quiet");
   disable("output:sanitize");
+  disable("output:numeric");
 
 // define some aliases for nodes classes
   alias("disc", "disk");
@@ -160,6 +165,12 @@ char **argv)
       validoption = true;
     }
 
+    if (strcmp(argv[1], "-numeric") == 0)
+    {
+      enable("output:numeric");
+      validoption = true;
+    }
+
     if(validoption)
     {	/* shift */
       memmove(argv+1, argv+2, (argc-1)*(sizeof(argv[0])));
